"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var native_renderer_1 = require("./native-renderer");
// import { Node as VirtNode } from "paperclip";
var events_1 = require("events");
var utils_1 = require("./utils");
var RenderEventTypes;
(function (RenderEventTypes) {
    RenderEventTypes["META_CLICK"] = "META_CLICK";
})(RenderEventTypes || (RenderEventTypes = {}));
var Renderer = /** @class */ (function () {
    function Renderer(protocol) {
        var _this = this;
        this.protocol = protocol;
        this.onMetaClick = function (listener) {
            _this._em.addListener(RenderEventTypes.META_CLICK, listener);
        };
        this._onStageMouseDown = function (event) {
            event.preventDefault();
            event.stopImmediatePropagation();
            var element = event.target;
            var targetId = _this._nativeNodeMap.get(element);
            if (element.nodeType !== 1 || !targetId || !event.metaKey)
                return;
            var virtNode = findVirtNodeById(targetId, _this._virtualRootNode);
            if (!virtNode)
                return;
            _this._em.emit(RenderEventTypes.META_CLICK, virtNode);
        };
        this._onStageMouseOver = function (event) {
            var element = event.target;
            var targetId = _this._nativeNodeMap.get(element);
            if (element.nodeType !== 1 || !event.metaKey || !targetId)
                return;
            var rect = element.getBoundingClientRect();
            Object.assign(_this._hoverOverlay.style, {
                display: "block",
                left: window.scrollX + rect.left + "px",
                top: window.scrollY + rect.top + "px",
                width: rect.width + "px",
                height: rect.height + "px"
            });
        };
        this._onStageMouseOut = function (event) {
            var element = event.target;
            if (element.nodeType !== 1)
                return;
            // const targetId = this._nativeNodeMap.get(element);
            Object.assign(_this._hoverOverlay.style, {
                display: "none"
            });
        };
        this._em = new events_1.EventEmitter();
        this._hoverOverlay = document.createElement("div");
        Object.assign(this._hoverOverlay.style, {
            position: "absolute",
            zIndex: 1024,
            display: "none",
            background: "rgba(124, 154, 236, 0.5)",
            width: "100px",
            height: "100px",
            pointerEvents: "none",
            top: "0px",
            left: "0px"
        });
        this._stage = document.createElement("div");
        this.mount = document.createElement("div");
        this.mount.appendChild(this._stage);
        this.mount.appendChild(this._hoverOverlay);
        this._stage.addEventListener("mousedown", this._onStageMouseDown, true);
        this._stage.addEventListener("mouseup", utils_1.preventDefault, true);
        this._stage.addEventListener("mouseover", this._onStageMouseOver);
        this._stage.addEventListener("mouseout", this._onStageMouseOut);
    }
    Renderer.prototype.handleEngineEvent = function (event) {
        // only accept events scoped to current file path
        if (event.kind !== "Evaluated" && event.file_path !== this._scopeFilePath) {
            return;
        }
        switch (event.kind) {
            case "Evaluated": {
                while (this._stage.childNodes.length) {
                    this._stage.removeChild(this._stage.childNodes[0]);
                }
                this._scopeFilePath = event.file_path;
                this._virtualRootNode = event.node;
                this._nativeNodeMap = new Map();
                var node = native_renderer_1.createNativeNode(event.node, this.protocol, this._nativeNodeMap);
                this._stage.appendChild(node);
            }
            case "Diffed": {
                // TODO
            }
        }
    };
    return Renderer;
}());
exports.Renderer = Renderer;
// TODO - move to paperclip lib
var findVirtNodeById = function (id, node) {
    if (node.id === id) {
        return node;
    }
    if (node.kind === "Element" || node.kind === "Fragment") {
        for (var _i = 0, _a = node.children; _i < _a.length; _i++) {
            var child = _a[_i];
            var ret = findVirtNodeById(id, child);
            if (ret)
                return ret;
        }
    }
    return null;
};
